#include <string.h>
#include <math.h>
#include "defined.h"


extern SENSOR_DATA gSensorData;
extern IMU_DATA gIMU_Data;
extern CNF_DATA gCnfData;
extern u_char gGPS_speed;

float Omega_I[3];
float Omega_P[3];
float Omega[3];
float gAccel[3];
float DCM_Matrix[3][3];

#define GYRO_DT 0.02 //20ms
#define ToRad(x) (x*0.01745329252)  // *pi/180
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void Vector_Scale(float *vectorOut, float *vectorIn, float scale2)
{
u_char c;
for(c=0; c<3; c++) vectorOut[c]=vectorIn[c]*scale2;
}
//---------------------------------------------------------------------------
void Vector_Sum(float vectorOut[3],float vectorIn1[3], float vectorIn2[3])
{
u_char c;
for(c=0; c<3; c++) vectorOut[c]=vectorIn1[c]+vectorIn2[c];
}
//---------------------------------------------------------------------------
float Vector_Dot_Product(float vector1[3],float vector2[3])
{ //Computes the dot product of two vectors
float op;
u_char c;

for(op=0, c=0; c<3; c++) op+=vector1[c]*vector2[c];
return op;
}
//---------------------------------------------------------------------------
void Vector_Cross_Product(float vectorOut[3], float v1[3],float v2[3])
{ //Computes the cross product of two vectors
vectorOut[0]= (v1[1]*v2[2]) - (v1[2]*v2[1]);
vectorOut[1]= (v1[2]*v2[0]) - (v1[0]*v2[2]);
vectorOut[2]= (v1[0]*v2[1]) - (v1[1]*v2[0]);
}
//---------------------------------------------------------------------------
void Matrix_Multiply(float matrixOut[3][3], float a[3][3], float b[3][3])
{ //Multiply two 3x3 matrixs.
u_char x, y, w;
float op[3];

for(x=0; x<3; x++)
  {
  for(y=0; y<3; y++)
    {
    for(w=0; w<3; w++) op[w]=a[x][w]*b[w][y];
    matrixOut[x][y]=op[0]+op[1]+op[2];
    }
  }
}
//---------------------------------------------------------------------------
void Accel_adjust(void)
{
#define N_EXPO_FILTER 10
u_char i;
float A[3];
float spd;

spd=gGPS_speed; spd/=(3.6*9.8); //(km/h)->(m/s)->(G/W)
A[0]=gSensorData.Accel[0];
A[1]=gSensorData.Accel[1]-(Omega[2]*spd);
A[2]=gSensorData.Accel[2]+(Omega[1]*spd);
for(i=0; i<3; i++) gAccel[i]=(A[i]+((N_EXPO_FILTER-1)*gAccel[i]))/N_EXPO_FILTER;
}
//---------------------------------------------------------------------------
/*
void CalcVibration(void)
{ // IIR HPF filter 5Hz
#define N_EF 10
float x0, y0;
static float x1=0, x2=0, y1=0, y2=0, v=0;
#define IIR_A0 0.638946
#define IIR_A1 -1.277891
#define IIR_A2 0.638946
#define IIR_B1 -1.142981
#define IIR_B2 0.412802

x0=sqrt(Vector_Dot_Product(gSensorData.Accel, gSensorData.Accel));
// The calculation of the vibration  (HPF IIR-filter)
y0=(IIR_A0*x0) + (IIR_A1*x1) + (IIR_A2*x2) - (IIR_B1*y1) - (IIR_B2*y2);
x2=x1; x1=x0; // shift x1 to x2, sample to x1
y2=y1; y1=y0; // shift y1 to y2, result to y1
if(y0<0) y0=-y0; //
v=(y0+((N_EF-1)*v))/N_EF;
gIMU_Data.Vibration=(u_char)(v*500);
}
*/
//---------------------------------------------------------------------------
void Normalize(void)
{
float err;
float tmp_m[3][3];
float len;
u_char i, problem;

err=-Vector_Dot_Product(DCM_Matrix[0], DCM_Matrix[1])*.5; //eq.19
Vector_Scale(tmp_m[0], DCM_Matrix[1], err); //X eq.19
Vector_Scale(tmp_m[1], DCM_Matrix[0], err); //Y eq.19
Vector_Sum(tmp_m[0], tmp_m[0], DCM_Matrix[0]); //X eq.19
Vector_Sum(tmp_m[1], tmp_m[1], DCM_Matrix[1]); //Y eq.19
Vector_Cross_Product(tmp_m[2], tmp_m[0], tmp_m[1]); // Z=XxY //eq.20
for(problem=0, i=0; i<3; i++)
  {
  len=sqrt(Vector_Dot_Product(tmp_m[i], tmp_m[i]));
  if(len>10 || len<0.1) { problem=1; break; }
  Vector_Scale(DCM_Matrix[i], tmp_m[i], 1/len);
  }
if(problem)
  {
  DCM_Matrix[0][0]=1; DCM_Matrix[0][1]=0; DCM_Matrix[0][2]=0;
  DCM_Matrix[1][0]=0; DCM_Matrix[1][1]=1; DCM_Matrix[1][2]=0;
  DCM_Matrix[2][0]=0; DCM_Matrix[2][1]=0; DCM_Matrix[2][2]=1;
  }
}
//---------------------------------------------------------------------------
void Drift_correction(void)
{ // Compensation the Roll, Pitch and Yaw drift.
float err_v[3];
float AccelNorm[3];
float Accel_magnitude;
float Accel_weight;
float v, v1;
static u_char cn_c;
u_char i;
static float Z_Matrix[3];

//----------- Roll and Pitch -------------
// Calculate the magnitude of the accelerometer vector
Accel_magnitude=sqrt(Vector_Dot_Product(gAccel, gAccel));
v=Accel_magnitude-1;
if(v>0.5) v=0.5; else if(v<-0.5) v=-0.5;
gIMU_Data.Overload=(s_char)(v*100);
if(v<0) v=-v;
Accel_weight=1 - (2*v); // Weight for accelerometer info (<0.5G = 0.0, 1G = 1.0 , >1.5G = 0.0)
gIMU_Data.Debug=(u_char)(Accel_weight*100);
//It requires the normalization of the vector Accel_Vector???
Vector_Scale(AccelNorm, gAccel, 1./Accel_magnitude);
// Delay data from matrix
for(i=0; i<3; i++) Z_Matrix[i]=(DCM_Matrix[2][i]+((N_EXPO_FILTER-1)*Z_Matrix[i]))/N_EXPO_FILTER;
//adjust the ground of reference
Vector_Cross_Product(err_v, AccelNorm, Z_Matrix);
//------------- Yaw -------------------
if(!(gIMU_Data.Flags & 0x80)) err_v[2]=-Omega[2]; // Yaw damping
else err_v[2]=0;
/*
v=sqrt(Vector_Dot_Product(gSensorData.Compass, gSensorData.Compass)); // Normalize
Vector_Scale(gSensorData.Compass, gSensorData.Compass, 1./v);
Vector_Cross_Product(tmp_v, DCM_Matrix[2], gSensorData.Compass); // West vector
err_v[2]=(tmp_v[0]*DCM_Matrix[1][1])-(tmp_v[1]*DCM_Matrix[1][0]); //ErrorYaw=WESTxY
*/
//--------
Vector_Scale(Omega_P, err_v, gCnfData.KP_RollPitch*Accel_weight);
// Calc error
v=sqrt(Vector_Dot_Product(err_v, err_v));
//------------------------------------------------------------
if(!(gIMU_Data.Flags & 0x80))
  { // Gyro calibrated, damping
  v1=v*1000;
  if(v<gCnfData.Accuracy) { if(++cn_c>=50) gIMU_Data.Flags|=0x80; return; }
  cn_c=0;
  Vector_Scale(err_v, err_v, gCnfData.KI_RollPitch*Accel_weight);
  Vector_Sum(Omega_I, Omega_I, err_v);
  }
else v1=v*100;
v1+=0.5; if(v1>100) v1=100;
gIMU_Data.ErrorGyro=(u_char)v1;  
}
//---------------------------------------------------------------------------
void Matrix_update(void)
{
float v_tmp[3];
float m_tmp1[3][3];
float m_tmp2[3][3];

Vector_Sum(v_tmp, gSensorData.Gyro, Omega_I); //adding Integrator term
Vector_Sum(Omega, v_tmp, Omega_P); //adding proportional term
m_tmp1[0][0]=1;
m_tmp1[0][1]=-GYRO_DT*Omega[2];//-z
m_tmp1[0][2]=GYRO_DT*Omega[1]; //y
m_tmp1[1][0]=GYRO_DT*Omega[2]; //z
m_tmp1[1][1]=1;
m_tmp1[1][2]=-GYRO_DT*Omega[0]; //-x
m_tmp1[2][0]=-GYRO_DT*Omega[1]; //-y
m_tmp1[2][1]=GYRO_DT*Omega[0];  //x
m_tmp1[2][2]=1;
Matrix_Multiply(m_tmp2, DCM_Matrix, m_tmp1); //a*b=c
memcpy(DCM_Matrix, m_tmp2, sizeof(DCM_Matrix) );
}
//---------------------------------------------------------------------------
//#define ACCEL_ANGLE_OUT
void Euler_angles(void)
{
#ifdef ACCEL_ANGLE_OUT         // Only accelerometer info (debugging purposes)
gIMU_Data.Roll=(s_int)(atan2(gAccel[1],gAccel[2])*180/M_PI); // atan2(acc_y,acc_z)
gIMU_Data.Pitch=(s_char)(-asin(gAccel[0])*180/M_PI);   // asin(acc_x)
gIMU_Data.Yaw=20;
#else
gIMU_Data.Roll=(s_int)(atan2(DCM_Matrix[2][1], DCM_Matrix[2][2])*180/M_PI);
gIMU_Data.Pitch=-(s_char)(asin(DCM_Matrix[2][0])*180/M_PI);
gIMU_Data.Yaw=-(s_int)(atan2(DCM_Matrix[1][0],DCM_Matrix[0][0])*180/M_PI);
#endif
}
//---------------------------------------------------------------------------
