#include <mega8.h>
#include <delay.h>
#include <string.h>
#include <math.h>
#include "defined.h"
#include "dcm.h"
#include "uart.h"

//---------------------------------------------------------------------------
SENSOR_DATA gSensorData;
IMU_DATA gIMU_Data;
CNF_DATA gCnfData;
eeprom CNF_DATA EEP_CnfData;
u_char gTWI_stat;
bit f_GyroUpdate;
bit f_AccelUpdate;
u_char gHostDbg;

u_char MPU6050_Init(void);
u_char MPU6050_GetData_Gyro(u_char f_calc);
u_char MPU6050_GetData_Acc(void);
//u_char HMC5883_Init();
//u_char HMC5883_GetData(void);
//---------------------------------------------------------------------------
void StopFromErros(u_char pos, u_char code, u_char stat)
{
u_char i, f;
ERR_DATA ErrData;

ErrData.pos=pos;
ErrData.code=code;
ErrData.stat=stat;
f=0;
while(1)
  {
  LED1=f; f=!f; LED2=f;
  send_UART(UART_ERR_DATA, (u_char *)&ErrData, sizeof(ERR_DATA),0);
  for(i=0; i<100; i++) { sending_UART(); delay_ms(10); }   
  }
}
//---------------------------------------------------------------------------
interrupt [EXT_INT0] void ext_int0_isr(void)
{ //ITG3205 data ready  10ms
u_char err;
static u_char div1=0;

#asm("sei") 
err=MPU6050_GetData_Gyro(div1);
if(err) StopFromErros(20, err, gTWI_stat);
if(div1==0)
  {
  if(ACC_DRDY)
    {
    err=MPU6050_GetData_Acc();
    if(err) StopFromErros(21, err, gTWI_stat);
    f_AccelUpdate=1;
    gIMU_Data.Flags|=1;
    }
  else if(f_AccelUpdate==1)
    {
    //err=HMC5883_GetData();                                    
    //if(err) StopFromErros(21, err, gTWI_stat);
    f_AccelUpdate=0;
    gIMU_Data.Flags|=2;
    }
  }    
if(div1) f_GyroUpdate=1;
div1=!div1;
}
//---------------------------------------------------------------------------
u_char gSPI_pt;
IMU_DATA gIMU_DataS;
bit f_SPI_end;
u_char gGPS_speed, gGPS_speedRAW;
#define SIZE_IMU_DATA (sizeof(IMU_DATA)*8)

#pragma optsize-
interrupt [EXT_INT1] void ext_int1_isr(void)
{// SPI Synchro
u_char ch, ind, m;

TCNT0=0x00;
if(f_SPI_end) return;
if(gSPI_pt==0) DDRD=0x12; // start sending
if(gSPI_pt<SIZE_IMU_DATA)
  { // send
  m=1; m<<=(gSPI_pt & 7);
  ind=gSPI_pt>>3; 
  ch=*(((u_char *)&gIMU_DataS)+ind);
  PORTD.4=(ch & m);
  }
else if(gSPI_pt==SIZE_IMU_DATA) DDRD=0x02; // start reading
else
  { // read
  m=1; m<<=(gSPI_pt-(SIZE_IMU_DATA+1)); 
  if(PIND.4) gGPS_speedRAW|=m;
  if(m==0x80) f_SPI_end=1;
  }
gSPI_pt++;      
}
#pragma optsize+
//---------------------------------------------------------------------------
void SPI_SetData(void)
{
u_char crc, i;
u_char *p;

if(!f_SPI_end) return;
if(TCNT0<4) return;
f_SPI_end=0;
gSPI_pt=0;
gIMU_DataS=gIMU_Data;
p=(u_char *)&gIMU_DataS;
for(i=0, crc=0xff; i<sizeof(IMU_DATA)-1; i++, p++) crc^=*p;
gIMU_DataS.CRC=crc;
gGPS_speed=gGPS_speedRAW;
gGPS_speedRAW=0;
} 
//---------------------------------------------------------------------------
void main(void)
{
u_char err, div5;
bit f_f;

// Input/Output Ports initialization
// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=Out Func1=Out Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=0 State1=0 State0=P 
PORTB=0x01;
DDRB=0x06;
// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State6=T State5=P State4=P State3=T State2=T State1=P State0=P 
PORTC=0x33;
DDRC=0x00;
// Port D initialization
// Func7=In Func6=In Func5=In Func4=Out Func3=In Func2=In Func1=Out Func0=In 
// State7=P State6=P State5=P State4=0 State3=P State2=P State1=0 State0=P 
PORTD=0xED;
DDRD=0x12;
// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 16,500 kHz
TCCR0=0x05; // 1T=~60mks
TCNT0=0x00;
// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;
// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;
// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: On
// INT1 Mode: Any change
GICR|=0xC0;
MCUCR=0x06;
GIFR=0xC0;
// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;
// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 56000
UCSRA=0x00;
UCSRB=0x98;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x11;
// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;
// 2 Wire Bus initialization
// Generate Acknowledge Pulse: On
// 2 Wire Bus Slave Address: 1h
// General Call Recognition: Off
// Bit Rate: 384,000 kHz
TWSR=0x00;
TWBR=0x0E;
TWAR=0x02;
TWCR=0x44;

if(EEP_CnfData.flash_ver!=_VERSION_)
  { // reset to default
  EEP_CnfData.flash_ver=_VERSION_;
  EEP_CnfData.KP_RollPitch=0.5;
  EEP_CnfData.KI_RollPitch=0.002;
  EEP_CnfData.Yaw_Damping=0.005;
  EEP_CnfData.Accuracy=0.002;
  EEP_CnfData.K_AccelGravity=K_ACCEL_GRAVITY; //247
  EEP_CnfData.K_GyroGain=K_GYRO_GAIN/2;       //0.0007
  }
gCnfData=EEP_CnfData;
//send_UART(UART_READ_CNF, (u_char *)&gCnfData, sizeof(CNF_DATA), 0);
//
err=MPU6050_Init(); if(err) StopFromErros(1, err, gTWI_stat);
delay_ms(1);
//err=HMC5883_Init(); if(err) StopFromErros(3, err, gTWI_stat);                      
//delay_ms(1);
f_SPI_end=1;
// test leds
delay_ms(100);
LED1=1;
delay_ms(100);
LED1=0; LED2=1;
delay_ms(100);
LED2=0;
delay_ms(100);
//
MPU6050_GetData_Gyro(0);
for(div5=0; div5<3; div5++) { gSensorData.Accel[div5]=1; gSensorData.Compass[div5]=1; } 
//
f_f=0; div5=0;
#asm("sei")
while (1)
  {
 SPI_SetData();
 if(f_GyroUpdate)
    {
    f_GyroUpdate=0;
    if(++div5>=5) div5=0;
    if(div5==0 || (gIMU_Data.Flags & 0x80)) { f_f=!f_f; LED1=f_f; }
    if(gIMU_Data.Overload>25 || gIMU_Data.Overload<-25) LED2=1;
    else LED2=0;
    Matrix_update();
    Accel_adjust();
    Normalize();
    Drift_correction();
    Euler_angles();
    if(gHostDbg & 1)    
      send_UART(UART_SENSOR_DATA, (u_char *)&gSensorData, sizeof(SENSOR_DATA), 0);
    else
      send_UART(UART_IMU_DATA, (u_char *)&gIMU_Data, sizeof(IMU_DATA),0);
    gIMU_Data.Flags&=0x80;
    }
  sending_UART();
  }
}
//---------------------------------------------------------------------------
