#include <mega8.h>
#include <delay.h>
#include "defined.h"

// TWCR bits
#define TWINT 0x80 // TWI Interrupt Flag
#define TWEA  0x40 // TWI Enable Acknowledge Bit
#define TWSTA 0x20 // TWI START Condition Bit
#define TWSTO 0x10 // TWI STOP Condition Bit
#define TWWC  0x8  // TWI Write Collision Flag
#define TWEN  0x4  // TWI Enable Bit
#define TWIE  0x1  // TWI Interrupt Enable
// TWI states
#define TWSR_START 0x8         // A START condition has been transmitted
#define TWSR_REP_START 0x10    // A repeated START condition has been transmitted
#define TWSR_SLA_W_OK 0x18     // SLA+W has been transmitted; ACK has been received
#define TWSR_SLA_W_ERR 0x20    // SLA+W has been transmitted; NOT ACK has been received
#define TWSR_DATA_TX_OK 0x28   // Data byte has been transmitted; ACK has been received
#define TWSR_DATA_TX_ERR 0x30  // Data byte has been transmitted; NOT ACK has been received
#define TWSR_A_LOST 0x38       // Arbitration lost in SLA+W or data bytes
#define TWSR_SLA_R_OK 0x40
#define TWSR_DATA_RX_ASK 0x50   
#define TWSR_DATA_RX_NASK 0x58   
//
#define TWI_PROGRESS { while ((TWCR & TWINT)==0); gTWI_stat=(TWSR & 0xF8); }

extern u_char gTWI_stat;
extern SENSOR_DATA gSensorData;
extern CNF_DATA gCnfData;
//---------------------------------------------------------------------------------------------------
u_char TWI_Begin(u_char addr, u_char reg)
{
TWCR=(TWINT | TWEN | TWSTA);  // Send START condition
TWI_PROGRESS;
if(gTWI_stat!=TWSR_START && gTWI_stat!=TWSR_REP_START)  return 1; // Check value of TWI Status Register.
TWDR=addr; // Load SLA_W into TWDR Register.
TWCR=(TWINT | TWEN); //Clear TWINT bit in TWCR to start transmission of address
TWI_PROGRESS;
if(gTWI_stat!=TWSR_SLA_W_OK) return 2;
TWDR=reg; // Load reg into TWDR Register.
TWCR=(TWINT | TWEN); //Clear TWINT bit in TWCR to start transmission of register
TWI_PROGRESS;
if(gTWI_stat!=TWSR_DATA_TX_OK) return 3;
return 0;
}
//---------------------------------------------------------------------------------------------------
u_char TWI_SendData(u_char addr, u_char reg, u_char *data, u_char len)
{
u_char err;
u_int i;
u_char *p;

err=TWI_Begin(addr, reg); if(err) return err;
for(p=data, i=0; i<len; i++, p++)
  {
  TWDR=*p; // Load DATA into TWDR Register.
  TWCR=(TWINT | TWEN); //Clear TWINT bit in TWCR to start transmission of data
  TWI_PROGRESS;
  if(gTWI_stat!=TWSR_DATA_TX_OK) return 4;
  }
TWCR=(TWINT | TWEN | TWSTO); // Send STOP condition
return 0;
}
//---------------------------------------------------------------------------------------------------
u_char TWI_SendByte(u_char addr, u_char reg, u_char data)
{
return TWI_SendData(addr, reg, &data, 1);
}
//---------------------------------------------------------------------------------------------------
/*
u_char TWI_RecData(u_char addr, u_char reg, u_char *data, u_char len)
{
u_char err;
u_char *p;

err=TWI_Begin(addr, reg); if(err) return err;
TWCR=(TWINT | TWEN | TWSTA);  // Send START condition
TWI_PROGRESS;
if(gTWI_stat!=TWSR_REP_START)  return 5; // Check value of TWI Status Register.
TWDR=(addr | 1); // Load SLA_R into TWDR Register.
TWCR=(TWINT | TWEN); //Clear TWINT bit in TWCR to start transmission of address
TWI_PROGRESS;
if(gTWI_stat!=TWSR_SLA_R_OK) return 6;
for(p=data; len>0; len--)
  {
  if(len==1) TWCR=(TWINT | TWEN);  // send NASK
  else TWCR=(TWINT | TWEN | TWEA); // send ASK
  TWI_PROGRESS;
  if((gTWI_stat==TWSR_DATA_RX_ASK) || (gTWI_stat==TWSR_DATA_RX_NASK))
    {
    *p++=TWDR;
    if(gTWI_stat==TWSR_DATA_RX_NASK) len=0;
    }
  else return 7;  
  }
TWCR=(TWINT | TWEN | TWSTO); // Send STOP condition
return 0;
}
*/
//---------------------------------------------------------------------------------------------------
u_char TWI_RecData(u_char addr, u_char reg, u_char *data, u_char l)
{
u_char err;
u_char *p;

err=TWI_Begin(addr, reg); if(err) return err;
TWCR=TWINT | TWSTA | TWEN;  // Send START condition
TWI_PROGRESS;
if(gTWI_stat!=TWSR_REP_START)  return 5; // Check value of TWI Status Register.
TWDR=(addr | 1); // Load SLA_R into TWDR Register.
TWCR=(TWINT | TWEN); //Clear TWINT bit in TWCR to start transmission of address
TWI_PROGRESS;
if(gTWI_stat!=TWSR_SLA_R_OK) return 6;

for(p=data; l>0; l--)
  {
  if(l==1) TWCR=(TWINT | TWEN);
  else TWCR=(TWINT | TWEN | TWEA);
  TWI_PROGRESS;
  if(gTWI_stat==TWSR_DATA_RX_ASK || gTWI_stat==TWSR_DATA_RX_NASK)
    {
    *p++=TWDR;
    if(gTWI_stat!=TWSR_DATA_RX_ASK) break;
    }
  else return 7;
  }
TWCR=TWINT | TWEN | TWSTO; // Send STOP condition
return 0;
}

//---------------------------------------------------------------------------------------------------
//--- MPU6050 ---
//---------------------------------------------------------------------------------------------------
#define MPU6050_ADDRESS 0x68       // 0x68  address pin AD0 low (GND)
//                                    0x69  address pin AD0 high (VCC)
//---------------------------------------------------------------------------------------------------
//defined(MPU6050_LPF_256HZ) 0    //Default settings LPF 256Hz/8000Hz sample
//defined(MPU6050_LPF_188HZ) 1
//defined(MPU6050_LPF_98HZ)  2
//defined(MPU6050_LPF_42HZ)  3
//defined(MPU6050_LPF_20HZ)  4
//defined(MPU6050_LPF_10HZ)  5
//defined(MPU6050_LPF_5HZ)   6

u_char MPU6050_Init(void)
{
u_char err;
err=TWI_SendByte(MPU6050_ADDRESS<<1, 0x6B, 0x80);             //PWR_MGMT_1    -- DEVICE_RESET 1
if(err) return err;
delay_ms(1000);
err=TWI_SendByte(MPU6050_ADDRESS<<1, 0x6B, 0x01);             //PWR_MGMT_1    -- SLEEP 0; CYCLE 0; TEMP_DIS 0; CLKSEL 3 (PLL with X Gyro reference)
if(err) return err;
err=TWI_SendByte(MPU6050_ADDRESS<<1, 0x19, 0x09);             //Fsample = 1kHz / (9 + 1) = 100Hz, or 10ms per sample
if(err) return err;
err=TWI_SendByte(MPU6050_ADDRESS<<1, 0x1A, 0x04);              //LPF_CFG = 0 => ACC bandwidth = 260Hz  GYRO bandwidth = 256Hz)
if(err) return err;                                            
err=TWI_SendByte(MPU6050_ADDRESS<<1,  0x1B, 0x18);             //GYRO_CONFIG   -- FS_SEL = 3: Full scale set to 2000 deg/sec
if(err) return err;
err=TWI_SendByte(MPU6050_ADDRESS<<1, 0x37,  0b10110000);       //INT_PIN_CFG   -- INT_LEVEL=1 ; INT_OPEN=0 ; LATCH_INT_EN=1 ; INT_RD_CLEAR=1 ; FSYNC_INT_LEVEL=0 ; FSYNC_INT_EN=0 ; I2C_BYPASS_EN=0 ; CLKOUT_EN=0
if(err) return err;
err=TWI_SendByte(MPU6050_ADDRESS<<1,  0x38, 0x01);             // DATA_RDY_EN Interrupt =1
if(err) return err;
err=TWI_SendByte(MPU6050_ADDRESS<<1,  0x1C, 0x10);             // ACCEL DATA_FORMAT; RANGE 8+-g
return err;

}
//---------------------------------------------------------------------------------------------------
u_char MPU6050_GetData_Gyro(u_char f_calc)
{
u_char buff[6];
u_char err, i, j;
static s_int raw_buff[3];
union { u_char ch[2]; s_int i; }da;
float fv;

err=TWI_RecData(MPU6050_ADDRESS<<1, 0x43, buff, 6);
if(err) return err;
for(i=0, j=0; i<3; i++, j+=2)
  {
  da.ch[1]=buff[j];
  da.ch[0]=(u_char)buff[j+1];
  if(!f_calc) raw_buff[i]=da.i;
  else
    {
    raw_buff[i]+=da.i;
    fv=(float)raw_buff[i];
    gSensorData.Gyro[i]=fv*gCnfData.K_GyroGain;
    }
  }
return 0;     
}
//---------------------------------------------------------------------------------------------------
//--- ACC ---
//---------------------------------------------------------------------------------------------------

#define ACC_OFFSET_X 0 //  0!!
#define ACC_OFFSET_Y 0 //  0!!
#define ACC_OFFSET_Z 0 //  0!!


u_char MPU6050_GetData_Acc(void)

{
u_char buff[6];
u_char err;
static s_int raw_buff[3];
union { u_char ch[2]; s_int i; }da;
float fv;
err=TWI_RecData(MPU6050_ADDRESS<<1, 0x3B, buff, 6);
if(err) return err;

  da.ch[1]=buff[0];
  da.ch[0]=(u_char)buff[1];
  raw_buff[0]=da.i>>4;
  fv=(float)raw_buff[0];
  gSensorData.Accel[0]=(fv+ACC_OFFSET_X)/gCnfData.K_AccelGravity;      // ---X
 
 da.ch[1]=buff[2];
  da.ch[0]=(u_char)buff[3];
  raw_buff[0]=da.i>>4;
  fv=(float)raw_buff[0];
  gSensorData.Accel[1]=(fv+ACC_OFFSET_Y)/gCnfData.K_AccelGravity;      // ---Y
                                                        
  da.ch[1]=buff[4];
  da.ch[0]=(u_char)buff[5];
  raw_buff[0]=da.i>>4;
  fv=(float)raw_buff[0];
  gSensorData.Accel[2]=(fv+ACC_OFFSET_Z)/gCnfData.K_AccelGravity;      // ---Z
    
 return 0;
} 
/*
//---------------------------------------------------------------------------------------------------
//---  HMC5883 ---
//---------------------------------------------------------------------------------------------------
#define  HMC5883_ADDRESS 0x1E
#define  HMC5883_OFFSET_X 25
#define  HMC5883_OFFSET_Y 71
#define  HMC5883_OFFSET_Z -10
//---------------------------------------------------------------------------------------------------
u_char HMC5883_Init()
{
u_char err;

err=TWI_SendByte(HMC5883_ADDRESS<<1, 0, 0x70);//ConfigRegA;8bt, 15Hz
if(err) return err;
err=TWI_SendByte(HMC5883_ADDRESS<<1, 0x2, 0x01);//ModeRegister; Single-Measurement.
return err;                                                
}
//---------------------------------------------------------------------------------------------------
u_char HMC5883_GetData(void)
{
u_char buff[6];
union { u_char ch[2]; s_int i; }da;
u_char err, i, j;
static u_char ind[]={ 0, 2, 1};
static s_int offset[]={ HMC5883_OFFSET_X, HMC5883_OFFSET_Z, HMC5883_OFFSET_Y };

err=TWI_RecData(HMC5883_ADDRESS<<1, 0x3, buff, 6);
if(err) return err;
for(i=0, j=0; i<3; i++, j+=2)
  {
  da.ch[1]=buff[j];
  da.ch[0]=(u_char)buff[j+1];
  da.i+=offset[i];
  gSensorData.Compass[ind[i]]=(float)da.i;
  }
err=TWI_SendByte(HMC5883_ADDRESS<<1, 0x2, 0x01);//ModeRegister; Single-Measurement.
return err;
}
//---------------------------------------------------------------------------------------------------
*/


