//----------------------------------------------------------------------
//--- UART -------------------------------------------------------------
//----------------------------------------------------------------------
#include <mega8.h>
#include <string.h>
#include "defined.h"
#include "uart.h"

#define SIZE_UART_BUFF_RX 32
#define SIZE_UART_BUFF_TX 64
//----------------------------------------------------------------------
extern CNF_DATA gCnfData;
extern IMU_DATA gIMU_Data;
extern eeprom CNF_DATA EEP_CnfData;
extern u_char gHostDbg;
//----------------------------------------------------------------------
u_char gSendUartCnt;
u_char gSendUartInd;
u_char gSendUartBuff[2+1+1+SIZE_UART_BUFF_TX+1];
u_char gRecUartCnt;
u_char gRecUartType;
u_char gRecUartInd;
u_char gRecUartState;
u_char gRecUartCRC;
u_char gRecUartBuff[SIZE_UART_BUFF_RX];
//----------------------------------------------------------------------
//----------------------------------------------------------------------
void sending_UART(void)
{
if(gSendUartCnt==0) return;
if((UCSRA & 0x20)==0) return;
UDR=gSendUartBuff[gSendUartInd];
gSendUartInd++;
if(gSendUartInd==gSendUartCnt) gSendUartCnt=0;
}
//----------------------------------------------------------------------
void send_UART(u_char type, u_char *buff, u_char cnt, u_char forse)
{
u_char crc, i;
u_char *p;

if(forse) while(gSendUartCnt) sending_UART();
if(gSendUartCnt!=0) return;
gSendUartCnt=0;
gSendUartInd=0;
p=gSendUartBuff;
*p++=SYN_UART1;
*p++=SYN_UART2;
*p++=cnt;
crc=cnt;
*p++=type;
crc^=type;
for(i=0; i<cnt; i++)
  {
  crc^=buff[i];
  *p++=buff[i];
  }
*p=crc;  
gSendUartCnt=2+1+1+cnt+1;
}
//----------------------------------------------------------------------
interrupt [USART_RXC] void usart_rx_isr(void)
{
u_char val;

val=UDR;
switch(gRecUartState)
  {
case 0: 
  if(val==SYN_UART1) gRecUartState++; 
break;
case 1: 
  if(val==SYN_UART2) gRecUartState++;
  else gRecUartState=0;
break;
case 2:
  if(val>SIZE_UART_BUFF_RX) { gRecUartState=0; break; }
  gRecUartCRC=val;
  gRecUartCnt=val;
  gRecUartState++;
break;
case 3:
  gRecUartCRC^=val;
  gRecUartType=val;
  gRecUartInd=0;
  gRecUartState++;
  if(gRecUartCnt==0) gRecUartState++;
break;  
case 4:
  gRecUartCRC^=val; 
  gRecUartBuff[gRecUartInd]=val;
  gRecUartInd++;
  if(gRecUartInd==gRecUartCnt) gRecUartState++;
break;    
case 5:
  gRecUartState=0;  
  if(val==gRecUartCRC) parsing_UART();
break;
  }
}
//----------------------------------------------------------------------
void parsing_UART(void)
{
switch(gRecUartType)
  {
  case UART_READ_CNF:
    if(gRecUartCnt!=0) break;
    send_UART(UART_READ_CNF, (u_char *)&gCnfData, sizeof(CNF_DATA),1);
  break;
  case UART_WRITE_CNF:
    if(gRecUartCnt!=sizeof(CNF_DATA)) break;
    memcpy(&gCnfData, gRecUartBuff, sizeof(CNF_DATA));
    gCnfData.flash_ver=_VERSION_;
    EEP_CnfData=gCnfData;
    send_UART(UART_WRITE_CNF, 0, 0, 1);
  break;
  case UART_COMMANDS:
    if(gRecUartCnt!=1) break;
    gIMU_Data.Flags&=0x7f;
    gHostDbg=gRecUartBuff[0];
    send_UART(UART_COMMANDS, 0, 0, 1);
  break;
  }
}
//------------------------------------------------------------------
//------------------------------------------------------------------
//------------------------------------------------------------------
